	
<!-- #include virtual="/quickstart/aspplus/include/header.inc" -->

<h4>Configuration File Format</h4>

ASP+ configuration files are XML-based text files  each named config.web  that can appear in any directory on an ASP+ Web 
Application Server.

Each config.web file applies configuration settings to the directory it is located within and all virtual child directories beneath it.   Settings in 
child directories can optionally override or modify settings specified in parent directories.  A root configuration file  
WinNT\ComPlus\&lt;version&gt;\config.web  provides default configuration settings for the entire machine.  
ASP+ configures IIS to prevent direct browser access to config.web files (ensuring that their values cant become public)  
returning 404: Forbidden errors when attempts are made to do so).
<p>
At runtime ASP+ uses these config.web configuration files to hierarchically compute a unique collection of settings for each incoming 
URL target request (note: these settings are only calculated once and then cached across subsequent requests  ASP+ automatically 
watches for file changes and will invalidate the cache if any of the configuration files change).
<p>
For example, the configuration settings for the URL: <u>http://myserver/myapplication/mydir/page.aspx</u> would be computed by applying 
config.web file settings in the following order:
<p>
<div class="code"><pre>
C:\WinNT\ComPlus\1710\config.web             &lt;= Base Configuration Settings for Machine
D:\MyApplication\config.web	                  &lt;= Overriden by Application Configuration Settings
D:\MyApplication\MyDir\config.web             &lt;= Overriden by Sub-Directory Config Settings
</pre></div>

If a config.web file is present at the root directory for a site, for example "Inetpub\wwwroot", those configuration settings will apply to every application in that site.  Note that the presence of a config.web file within a given directory or application root is completely optional.  If a config.web file is not present, then all configuration settings for the directory are automatically inherited from the parent directory.

<h5>Configuration Section Handlers and Sections</h5>

A config.web configuration file is an XML-based text file that can contain standard XML document elements  
including well-formed tags, comments, text, cdata, etc.  The file may ANSI, UTF-8, or Unicode -- the system will automatically detect the encoding.

The root element of a config.web file is always a <configuration> tag.  ASP+ and end-user settings are then encapsulated within this 
tag:

<div class="code"><pre>
&lt;configuration&gt;
    &lt;! Configuration Settings Would Go Here --&gt;
&lt;/configuration&gt;
</pre></div>

The &lt;configuration&gt; tag typically contains two different types of elements: 1) configuration section handler declarations, and 
2) configuration section settings.
<p>

<ul>

<li><b>Configuration Section Handlers</b>  - The ASP+ configuration infrastructure makes no assumptions regarding the file-format or supported settings within a config.web file.  
Instead, it delegates the processing of config.web data to "configuration section handlers"  NGWS runtime classes that implement the 
IConfigurationSectionHandler interface.  An individual IConfigurationSectionHandler declaration only needs to appear once, typically at the 
machines root config.web file. The config.web files in child directories automatically inherit this declaration.
Configuration section handlers are declared within a config.web file using "add" tag directives nested within a containing 
&lt;configsections&gt; tag.  Each add tag identifies a tag name denoting a specific section of configuration data, and an 
associated IConfigurationSectionHandler class that processes it.  
<p>

<li><b>Configuration Sections</b> - ASP+ configuration settings are represented within "configuration tag sections", also nested within a &lt;configuration&gt; tag.  For each
configuration section, an appropriate section handler must be defined in the config hierarchy.  For example, in the sample below the 
tag &lt;httpmodules&gt; is the configuration section that defines the "httphandlers" configuration data.  It is the 
System.Web.Configuration.HttpModulesConfigurationHandler class that is responsible for interpreting the content contained 
within the &lt;httpmodules&gt; tag at runtime.  

</ul>

<p>
<div class="code"><pre>
&lt;configuration&gt;

    &lt;configsections&gt;
        &lt;add name="httpmodules" type="System.Web.Configuration.HttpModulesConfigurationHandler" /&gt;
    &lt;/configsections&gt;

    &lt;httpmodules&gt;
        &lt;add type="System.Web.SessionState.CookielessSessionModule" /&gt;
        &lt;add type="System.Web.Caching.OutputCacheModule" /&gt;
        &lt;add type="System.Web.SessionState.SessionStateModule" /&gt;
        &lt;add type="System.Web.Security.WindowsAuthenticationModule" /&gt;
        &lt;add type="System.Web.Security.CookieAuthenticationModule" /&gt;
        &lt;add type="System.Web.Security.PassportAuthenticationModule" /&gt;
        &lt;add type="System.Web.Security.CustomAuthenticationModule" /&gt;
        &lt;add type="System.Web.Security.UrlAuthorizationModule" /&gt;
        &lt;add type="System.Web.Security.FileAuthorizationModule" /&gt;
    &lt;/httpmodules&gt;

&lt;/configuration&gt;
</pre></div>

<h5>Standard ASP+ Configuration Section Handlers</h5>

ASP+ ships with a number of standard configuration section handlers that are used to process configuration settings within 
config.web files.  Each section is briefly described below, along with a pointer to where more information can be found.
<p>

<table class="table2" width="90%" cellpadding=3>
<tr>
<th>Section Name</th><th width="100%">Description</th>
</tr>
<tr>
<td><b>&lt;httpmodules&gt;</b></td><td>Responsible for configuring Http Modules within an application. Http Modules pariticipate in the processing of every request into an application, and common uses include security and logging.</td>
</tr>
<tr>
<td><b>&lt;httphandlers&gt;</b></td><td>Responsible for mapping incoming URLs to IHttpHandler classes. Sub-directories do not inherit these settings.
Also responsible for mapping incoming URLs to IHttpHandlerFactory classes.  Data represented in &lt;httphandlerfactories&gt; sections are hierarchically inherited by sub-directories.  For more information about this configuration section, refer to the <a href="httphandlers.aspx">Http Handlers and Factories</a> section of this tutorial.</td>
</tr>
<tr>
<td><b>&lt;sessionstate&gt;</b></td><td>Responsible for configuring the session state HttpModule.  For more information about this configuration section, refer to the <a href="stateoverview.aspx">Managing Application State</a> section of this tutorial.</td>
</tr>
<tr>
<td><b>&lt;globalization&gt;</b></td><td>Responsible for configuring the globalization settings of an application.  For more information about this configuration section, refer to the <a href="internationalization.aspx">Localization</a> section of this tutorial.</td>
</tr>
<tr>
<td><b>&lt;compilation&gt;</b></td><td>Responsible for all compilation settings used by ASP+.  For more information about this configuration section, refer to the <a href="businessobjs.aspx">Business Objects</a> and <a href="debugcomsdk.aspx">Debugging</a> sections of this tutorial.</td>
</tr>
<tr>
<td><b>&lt;trace&gt;</b></td><td>Responsible for configuring the ASP+ trace service. For more information about this configuration section, refer to the <a href="tracingoverview.aspx">Tracing</a> section of this tutorial.</td>
</tr>
<tr>
<td><b>&lt;security&gt;</b></td><td>Responsible for all security settings used by the ASP+ security HttpModule.  For more information about this configuration section, refer to the <a href="securityoverview.aspx">Security</a> section of this tutorial.</td>
</tr>
<tr>
<td><b>&lt;iisprocessmodel&gt;</b></td><td>Responsible for configuring the ASP+ process model settings on IIS Web Server Systems.  </td>
</tr>
<tr>
<td><b>&lt;browsercaps&gt;</b></td><td>Responsible for controlling the settings of the browser capabilities component.  For more information about this configuration section, refer to the <a href="configretrieve.aspx">Retrieving Configuration</a> section of this tutorial.</td>
</tr>
</table>

	<!-- #include virtual="/quickstart/aspplus/include/footer.inc" -->