<%@ Register TagPrefix="Acme" TagName="SourceRef" Src="/quickstart/util/SrcRef.aspx"%>

<!-- #include virtual="/quickstart/aspplus/include/header.inc" -->

<h4>Forms-based Authentication</h4>

<p>
Forms-based authentication is an ASP+ authentication service that allows applications to provide their own login UI and do their own credential verification.  ASP+ handles authenticating users, redirecting unauthenticated users to the login page, and all the necessary cookie management.  This sort of authentication is a popular technique, used by many web sites.
<p>
In order to use forms-based authentication, an application has to be configured to use this authentication scheme.  This is by done setting the &lt;authentication&gt; to "Cookie", and denying access to anonymous users, as the following example shows (this is in the config.web file for the desired application):

<div class="code"><xmp>
<configuration>
    <security>
        <authentication mode="Cookie"/>
        <authorization>
            <deny users="?" /> 
        </authorization>
    </security>
</configuration>
</xmp></div>
<p>
Forms-based authentication allows administrators to configure the name of the cookie to use, the key with which to encrypt it, and the URL to use for the login page.  The following table shows the valid attributes for the &lt;cookie&gt element, which is a subelement of &lt;authentication&gt; element as shown below.

<div class="code"><xmp>
<security>
    <authentication mode="Cookie">
        <cookie decryptionkey="autogenerate" loginurl="login.aspx" cookie=".ASPXCOOKIEDEMO" />	
    </authentication>
</security>
</xmp></div>

<table class="table2" cellpadding=3 width="95%">
<tr>
	<th width="150"><b>Attribute</b></th>
	<th><b>Description</b></th>
</tr>
<tr>
	<td>decryptionkey</td>
	<td>This specifies the key to use for encrypting/decrypting the cookies for authentication.  If this value is omitted or if "autogenerate" is used, ASP+ uses a system specific key culled from the Crypto APIs.  If "autogenerate" is used, these cookies cannot be used across a web farm, since each key will be machine specific.  If web farm or multi-machine support is desired, administrators need to provide a key, stored in cleartext.</td>
</tr>
<tr>
	<td>loginurl</td>
	<td>The login URL to which unauthenticated users are redirected.  This can be on the same machine, or a remote one.  If it's on a remote machine, both machines need to be using the same value for the decryptionkey attribute.</td>
</tr>
<tr>
	<td>cookie</td>
	<td>The name of the HTTP cookie to use for authentication purposes.  Note that if more than one application wants to used forms-based authentication services on a single machine, they should each configure a unique cookie value.  In order to avoid casing-dependencies in URL's, ASP+ uses '/' as the Path value when setting authentication cookies, so they're sent back to every application on the site.</td>
</tr>

</table>
<p>
After the application has been configured, a login page needs to be provided.  The following example shows a simple example of a login page.  When the sample is run, it requests the default.aspx page.  Unauthenticated requests will be redirected to the login page (login.aspx) which presents a simple forms that prompts for an email address and a password (use Username="jdoe@somewhere.com" and Password="password" as credentials).

After validating the credentials, the application calls the following:

<div class="code"><xmp>
CookieAuthentication.RedirectFromLoginPage(UserEmail.Value, PersistCookie.Checked);
</xmp></div>

This redirects the user back to the original URL they requested.  Applications that don't wish to perform the redirection can call either <b>CookieAuthentication.GetAuthCookie</b> to retrieve the cookie value or <b>CookieAuthentication.SetAuthCookie</b> to attach a properly encrypted cookie to the outgoing response.  These techniques can be useful for applications that wish to provided login UI embedded into a containing page or that wish to have more control over where users are redirected.  Authentication cookies can either be temporary or permanent.  Temporary cookies last only the duration of the current browser session (when the browser is closed, the cookie is lost).  Permanent cookies are saved by the browser and are sent back across browser sessions unless explicitly deleted by the user.

<p>

<Acme:SourceRef 
  RunSample="/quickstart/aspplus/samples/security/CookieAuth/default.aspx" 
  ViewSource="/quickstart/aspplus/samples/security/cookieauth.src"
  Icon="/quickstart/aspplus/images/formsauth.gif"
  Caption="Forms-Based/Cookie Authentication"
  runat="server" />

<p>
The authentication cookie used by forms-authentication consists of a linearized version of the <b>System.Web.Security.CookieAuthenticationTicket</b> class.  The information includes the user name (but not the password), the version of forms authentication used, the date the cookie was issued, and a field for optional application specific data.
<p>
Application code can revoke/remove authentication cookies using the <b>CookieAuthentication.SignOut</b> method.  This will remove the authentication cookie regardless or whether it's a temporary or permanent cookie.
<p>
It's also possible to supply the forms-based authentication services with a list of valid credentials using configuration.
<p>
<div class="code"><xmp>
<security>
    <authentication>
        <credentials passwordformat=SHA1 >
            <user name=Mary password=GASDFSA9823598ASDBAD/>
            <user name=John password=ZASDFADSFASD23483142/>	
        </credentials>
    </authentication>
</security>
</xmp></div>
<p>
The application can then call <b>CookieAuthenticationManager.Authenticate</b>, supplying the username and password, and ASP+ will perform the credential verification.  Credentials can be stored in cleartext, or as SHA1 or MD5 hashes, according to the following values of the passwordformat attribute.
<p>
<table class="table2" cellpadding=3>
	<tr>
		<th width="150">Hash Type</th>
		<th>Description</th>
	</tr>
	<tr>
		<td>Clear</td>
		<td>Passwords are stored in cleartext</td>
	</tr>
	<tr>
		<td>SHA1</td>
		<td>Passwords are stored as SHA1 digests</td>
	</tr>
	<tr>
		<td>MD5</td>
		<td>Passwords are stored as MD5 digests</td>
	</tr>
</table>


<!-- #include virtual="/quickstart/aspplus/include/footer.inc" -->