
<%@ Register TagPrefix="Acme" TagName="SourceRef" Src="/quickstart/util/SrcRef.aspx"%>

<!-- #include virtual="/quickstart/aspplus/include/header.inc" -->

<h4>Trace Logging to Page Output</h4>

Page-level Tracing provides the ability to write debugging statements directly to a page's output, and conditionally execute 
debugging code when tracing is enabled.  To enable Tracing for a page, include the following directive at the top of the page code:
<p>

<div class="code"><pre>
&lt;%@ Page Trace="true"%&gt;
</pre></div>

Trace statements may also be organized by category using the <b>TraceMode</b> attribute of the Page directive.  
If no TraceMode attribute is defined, the default value is "SortByTime".

<div class="code"><pre>
&lt;%@ Page Trace="true" TraceMode="SortByCategory" %&gt;
</pre></div>

<p>
The following example shows the default output when Page-level Tracing is enabled.  Note that ASP+ inserts timing information for
important places in the page's execution lifecycle:
<p>

<Acme:SourceRef 
  RunSample="/quickstart/aspplus/samples/trace/trace1.aspx" 
  ViewSource="/quickstart/aspplus/samples/trace/trace1.src"
  Icon="/quickstart/aspplus/images/pagetrace1.gif"
  Caption="Trace1.aspx"
  runat="server" />

<p>
The Page exposes a <b>Trace</b> property (of type "TraceContext") which can be used to output debugging statements to the page output, provided tracing is enabled.
Using the TraceContext, you can write debugging statements using the <b>Trace.Write</b> and <b>Trace.Warn</b> methods, which each take category
and message strings as arguments.  Trace.Warn statements are identical to Trace.Write statements, except they are output in <span style="color:red">red</span>.

<div class="code"><pre>
Trace.Write("Custom Trace","Beginning User Code...");
...
Trace.Warn("Custom Trace","Array count is null!");
</pre></div>

<p>
When Tracing is disabled (i.e. Trace="false" on the page directive, or not present), these statements will not execute and no trace output
will appear in the client browser.  This makes it possible to keep debugging statements in production code and enable them conditionally
at a later time. 

<p>
Often you will need to execute additional code to construct the statements to pass to the Trace.Write or Trace.Warn methods, where this 
code should only execute if tracing is enabled for the page.  To support this, the Page exposes a boolean property, 
<b>Trace.IsEnabled</b>, which returns true only if tracing is enabled for the page.  You should check this property first to guarantee your 
debugging code will only execute when tracing is on.

<div class="code"><pre>
if (Trace.IsEnabled)
{
    for (int i=0; i&lt;ds.Tables["Categories"].Rows.Count; i++)
    {
        Trace.Write("ProductCategory",ds.Tables["Categories"].Rows[i][0].ToString());
    }
}
</pre></div>

<p>
The following example shows the use of Trace.Write and Trace.Warn to output debugging statements.  Also note the use of the Trace.IsEnabled
property to conditionally execute extra debugging code.  In this example, we have sorted the Trace information by category.
<p>

<Acme:SourceRef 
  RunSample="/quickstart/aspplus/samples/trace/trace2.aspx" 
  ViewSource="/quickstart/aspplus/samples/trace/trace2.src"
  Icon="/quickstart/aspplus/images/pagetrace2.gif"
  Caption="Trace2.aspx"
  runat="server" />

<p>
ASP+ also provides a way to enable Tracing for the entire application (not just a single page).  To read more about application-level 
tracing, click <a href="tracelogapp.aspx">here</a>.

<h4>Section Summary</h4>
<ol>
<li>Page-level tracing is enabled using a Trace="true" attribute on the top-level Page directive.
<li>Page-level tracing allows developers to write debugging statements as part of a page's client output.  Trace statements are output using the Trace.Write and Trace.Warn methods, passing a category and message for each statement.
<li>Debugging code may be conditionally executed depending on whether Trace is enabled for the page.  Use the Trace.IsEnabled property of the Page to determine whether tracing is enabled.
</ol>


<!-- #include virtual="/quickstart/aspplus/include/footer.inc" -->