
<%@ Register TagPrefix="Acme" TagName="SourceRef" Src="/quickstart/util/SrcRef.aspx"%>

<!-- #include virtual="/quickstart/aspplus/include/header.inc" -->

<h4>Data Access and Customization</h4>
<p>

<!-- #include virtual="/quickstart/aspplus/include/wftoc2-1.inc" -->
<p>
<hr>

<!--BEGIN SECTION-->
<a name="templates">
<span class="subhead">Introduction to Templated Controls</span>
<p>
While the DataGrid server control demonstrated in the previous section is suitable for many web application secnarios where a 
grid-like representation of data is appropriate, many times the presentation of data needs to be much more rich.  ASP+ offers 
two controls, <b>DataList</b> and <b>Repeater</b>, which give the developer greater flexibility over the rendering of list-like 
data.  We call these controls "look-less", because they have no default rendering of their own.  The way data is rendered is 
completely determined by the developer's implementation of the control's <b>templates</b>, which describe how to present
data items.
<p>
Like the DataGrid, DataList and Repeater support a <b>DataSource</b> property, which can be set to any ICollection type.  The data in this 
DataSource is bound to the control using the it's <b>DataBind</b> method.  Once the data is bound, the format of each data item is 
described by a <b>template</b>.  
<p>
The <b>ItemTemplate</b> controls the rendering of each item in the DataSource collection.  Inside an ItemTemplate, we can define 
any arbitrary presentation code (HTML or otherwise).  Using the ASP+ databinding syntax, we can insert values from the data bound to 
the DataList or Repeater control.  For example:
<p>

<div class="code"><pre>
&lt;ASP:Repeater id="MyRepeater" runat="server"&gt;

    &lt;template name="itemtemplate"&gt;
        Hello &lt;%# DataBinder.Eval(Container.DataItem, "name") %&gt; !
    &lt;/template&gt;

&lt;/ASP:Repeater&gt;
</pre></div>

<p>
The <b>Container</b> represents the first control in the immediate hierarchy that supports the 
<b>System.Web.UI.INamingContainer</b> marker interface.  In this case, the Container resolves to an object of type 
<b>System.Web.UI.WebControls.RepeaterItem</b>, which has a <b>DataItem</b> property.  As the Repeater iterates
over the DataSource collection, the DataItem contains the current item in this collection.  For example, if the DataSource is set to 
an ArrayList of Employee objects, the DataItem will be of type Employees.  When bound to a DataView, the DataItem is of type
<b>DataRowView</b>.
<p>
The following example demonstrates a Repeater control bound to a DataView (returned from a SQL query).  We have also defined a
<b>HeaderTemplate</b> and <b>FooterTemplate </b> which render at the beginning and end of the list, respectively.
<p>

<Acme:SourceRef 
  RunSample="/quickstart/aspplus/samples/webforms/customize/datalist1.aspx" 
  ViewSource="/quickstart/aspplus/samples/webforms/customize/datalist1.src"
  Icon="/quickstart/aspplus/images/datalist1.gif"
  Caption="Repeater1.aspx"
  runat="server" />

<p>
The Repeater control just iterates over the bound data, rendering the itemtemplate once for each item in the DataSource collection.
It does not render anything besides the elements contained in its templates.  While the Repeater is a general purpose iterator, 
the DataList provides some additional features for controlling the layout of the list.  Unlike the Repeater, DataList renders additonal
elements, like table rows and cells and spans containing style attributes, outside of the template definition to enable this richer 
formatting.  For example, DataList supports <b>RepeatColumns</b> and <b>RepeatDirection</b>properties that specify whether 
data should be rendered in multiple columns, and in which direction (vertical or horizontal) to render the data items.  DataList also
supports style attributes:
<p>

<div class="code"><pre>
&lt;ASP:DataList runat="server" DataSource="&lt;%#MyData%&gt;" 
    RepeatColumns="2" 
    RepeatDirection="Horizontal" 
    ItemStyle-Font-Size="10pt" 
    ItemStyle-Font-Name="Verdana"
&gt;
    ...
&lt;/ASP:DataList&gt;
</pre></div>

<p>
<b>Note: </b> For the remainder of this section we concentrate on the many features of the DataList control.  For more information about the 
Repeater control, refer to the <a href="/quickstart/aspplus/samples/webforms/ctrlref/webctrl/repeater/doc_repeater.aspx">Repeater</a> 
section of the <a href="webcontrolsref.aspx">Web Forms Controls Reference</a> section of this tutorial.
<p>

The following sample demonstrates the use of the DataList control.  Notice that we've changed the look of the data items 
from the previous example by simply changing the contents of the control's ItemTemplate.  The RepeatDirection and RepeatColumns 
properties determine how the ItemTemplates are laid out:
<p>

<Acme:SourceRef 
  RunSample="/quickstart/aspplus/samples/webforms/customize/datalist2.aspx" 
  ViewSource="/quickstart/aspplus/samples/webforms/customize/datalist2.src"
  Icon="/quickstart/aspplus/images/datalist2.gif"
  Caption="Datalist2.aspx"
  runat="server" />

<p>
To drive home the idea that template's are infinitely flexible, consider the next example, which changes the itemtemplate once again.
This time, we've substituted the "src" attribute of an &lt;img&gt; tag with one of the DataItem's values.  We've also used the 
<b>Format</b> String parameter of <b>DataBinder.Eval</b> to substitute a DataItem value in the querystring for a URL.
<p>

<Acme:SourceRef 
  RunSample="/quickstart/aspplus/samples/webforms/customize/datalist3.aspx" 
  ViewSource="/quickstart/aspplus/samples/webforms/customize/datalist3.src"
  Icon="/quickstart/aspplus/images/datalist3.gif"
  Caption="Datalist3.aspx"
  runat="server" />

<!--BEGIN SECTION-->
<br>
<a name="postback">
<br>
<span class="subhead">Handling Postbacks from a Template</span>
<p>
Like the DataGrid, you can fire a <b>Command</b> from inside a DataList template that is passed to an event handler wired to the 
DataList itself.  For example, a LinkButton inside the itemtemplate might fire a <b>Select</b> command.  By setting the 
<b>OnSelectedIndexChanged</b> property of the DataList, we can call an event handler in response to this command.

<p>

<div class="code"><pre>
&lt;ASP:DataList id="MyDataList" OnSelectedIndexChanged="MyDataList_Select" runat="server"&gt;

    &lt;template name="itemtemplate"&gt;

        &lt;asp:linkbutton Command="Select" runat="server"&gt;
            &lt;%# DataBinder.Eval(Container.DataItem, "title") %&gt;
        &lt;/asp:linkbutton&gt;

    &lt;/template&gt;

&lt;/ASP:DataList&gt;
</pre></div>

<p>
To see this code in action, view the sample below.  In the MyDataList_Select event handler we populate several other server controls
with the details about the particular selected item.
<p>


<Acme:SourceRef 
  RunSample="/quickstart/aspplus/samples/webforms/customize/datalist4.aspx" 
  ViewSource="/quickstart/aspplus/samples/webforms/customize/datalist4.src"
  Icon="/quickstart/aspplus/images/datalist4.gif"
  Caption="Datalist4.aspx"
  runat="server" />

<p>
Note that while the DataList recognizes a few special commands such as Select and Edit/Update/Cancel, the Command String fired inside a template 
can be any arbitrary String.  For all commands, the DataList's <b>OnItemCommand</b> is fired.  We can wire this event to a handler
as in the previous example:

<p>
<div class="code"><pre>
&lt;script runat="server"&gt;

    protected void MyDataList_ItemCommand(Object Src, DataListCommandEventArgs E ) 
    {
        String command = ((LinkButton)E.CommandSource).Text;

        switch(command) 
        {
          case "Discussions" :  ShowDiscussions((DataRowView)E.Item.DataItem);  break;
          case "Ratings"     :  ShowRatings((DataRowView)E.Item.DataItem);      break;
        }
    }

&lt;/script&gt;

&lt;ASP:DataList id="MyDataList" OnItemCommand="MyDataList_ItemCommand" runat="server"&gt;

    &lt;template name="itemtemplate"&gt;

        &lt;asp:linkbutton Command="Ratings" runat="server"&gt;
            View Ratings
        &lt;/asp:linkbutton&gt;
        |
        &lt;asp:linkbutton Command="Discussions" runat="server"&gt;
            View Discussions
        &lt;/asp:linkbutton&gt;

    &lt;/template&gt;

&lt;/ASP:DataList&gt;
</pre></div>
<p>
Notice that because more than one Command can fire this event handler, we must employ a switch statement to determine the 
particular Command that was fired.  To see this code in action, view the sample below:
<p>

<Acme:SourceRef 
  RunSample="/quickstart/aspplus/samples/webforms/customize/datalist5.aspx" 
  ViewSource="/quickstart/aspplus/samples/webforms/customize/datalist5.src"
  Icon="/quickstart/aspplus/images/datalist5.gif"
  Caption="Datalist5.aspx"
  runat="server" />

<!--BEGIN SECTION-->
<br>
<a name="selectandedit">
<br>
<span class="subhead">Using Select and Edit Templates</span>
<p>

In addition to handling the Select Command using a page-level event handler, the DataList can respond to this event internally.  If a 
<b>SelectedItemTemplate</b> is defined for the DataList, the DataList will render this template for the item that fired the Select 
Command.  In this next example we use the SelectedItemTemplate to embolden the title of the selected book:
<p>

<Acme:SourceRef 
  RunSample="/quickstart/aspplus/samples/webforms/customize/datalist6.aspx" 
  ViewSource="/quickstart/aspplus/samples/webforms/customize/datalist6.src"
  Icon="/quickstart/aspplus/images/datalist6.gif"
  Caption="Datalist6.aspx"
  runat="server" />

<p>
DataList also supports an <b>EditItemTemplate</b> for rendering an item whose index is equal to the DataList's <b>EditItemIndex</b> 
property.  For details about how editing and updating works, refer the the <a href="webdataaccess.aspx#update">Updating Data</a> 
topic of the <a href="webdataaccess.aspx">Data Access</a> section of this tutorial.
<p>

<Acme:SourceRef 
  RunSample="/quickstart/aspplus/samples/webforms/customize/datalist7.aspx" 
  ViewSource="/quickstart/aspplus/samples/webforms/customize/datalist7.src"
  Icon="/quickstart/aspplus/images/datalist7.gif"
  Caption="Datalist7.aspx"
  runat="server" />


<!--BEGIN SECTION-->
<br>
<a name="finding">
<br>
<span class="subhead">Finding a Control Inside a Template</span>
<p>

Sometimes it is necessary to locate a control contained inside a template.  If a control is given an ID in a template, that control may be
retrieved from its Container (the first control in the parent hierarchy that supports <b>INamingContainer</b>).  In this case, the 
Container is the DataListItem control.  Note that even though there are several controls with the same ID (by virtue of the DataList's 
repetition), each is contained logically in the namespace of the DataListItem Container control.
  
<p>
We can go through the DataList's <b>Items</b> collection to retrieve the DataListItem for a given index, and then call 
the DataListItem's <b>FindControl</b> method (inherited from the base Control class) to retrieve a control with a particular ID.
<p>

<div class="code"><pre>
&lt;script runat="server"&gt;

    public void Page_Load(Object sender, EventArgs E))
    {
        // set datasource and call databind here

        for (int i=0; i&lt;MyDataList.Items.Count; i++)
        {
           String isChecked = ((CheckBox) MyDataList.Items[i].FindControl("Save")).Checked.ToString();
        }
    }
&lt;/script&gt;

&lt;ASP:DataList id="MyDataList" runat="server"&gt;

    &lt;template name="itemtemplate"&gt;
         &lt;asp:CheckBox id="Save" runat="server"/&gt; &lt;b&gt;Save to Favorites&lt;/b&gt;
    &lt;/template&gt;

&lt;/ASP:DataList&gt;
</pre></div>

<p>
To see how this works, view the sample below:
<p>

<Acme:SourceRef 
  RunSample="/quickstart/aspplus/samples/webforms/customize/datalist8.aspx" 
  ViewSource="/quickstart/aspplus/samples/webforms/customize/datalist8.src"
  Icon="/quickstart/aspplus/images/datalist8.gif"
  Caption="Datalist8.aspx"
  runat="server" />

<!--BEGIN SECTION-->
<a name="endofsection">

<h4>Section Summary</h4>
<ol>
<li>The DataList and Repeater controls provide developers fine-tuned control over the rendering of databound lists.
<li>Rendering of bound data is controlled using a template, such as the HeaderTemplate, FooterTemplate, or ItemTemplate.
<li>The Repeater control is a general-purpose iterator, and does not inject anything in its rendering that is not contained in a template.
<li>The DataList control offers more control over the layout and style of items, and outputs it's own rendering code for formatting.
<li>The DataList supports the Select, Edit/Update/Cancel, and Item Command events, which may be handled at the page-level by
wiring event handlers to the DataList's Command events.
<li>DataList supports a SelectedItemTemplate and EditItemTemplate for control over the rendering of a selected or editable item.
<li>Controls may be programmatically retrieved from a template using the Control.FindControl method.  This should be called on
a DataListItem retrieved from the DataList's Items collection.
</ol>
<p>

<!-- #include virtual="/quickstart/aspplus/include/footer.inc" -->