
<!-- #include virtual="/quickstart/aspplus/include/header.inc" -->

<h4>Introducing Web Services</h4>

<P>
The Internet is quickly evolving from today's Web sites that just deliver UI Pages to browsers to a next generation of programmable Web sites that directly link organizations, applications, services, and devices with one another.  These programmable Web sites become more than passively accessed sites - they become reusable, intelligent Web Services.
</P>
<P>
The NGWS runtime provides built-in support for creating and exposing Web Services  using a programming abstraction that is consistent and familiar to both ASP+ Web Forms developers and existing VB users.  The resulting model is both scalable and extensible  and embraces open Internet standards (HTTP, XML, SOAP, SDL) so that it can be accessed and consumed from any client or Internet enabled device.
</P>
<H5>ASP+ Web Services</H5>
<P>
ASP+ provides support for Web Services with the .asmx file. An .asmx file is a text file that is similar
to an .aspx file. These files can be part of an ASP+ application that includes .aspx files. These files are then URI-addressable, just as .aspx are.
</P>
<P>
The following is a very simple example of an .asmx file:
</P>

<div class="code"><pre>
&lt;%@ WebService Language=C# %&gt;

using System.Web.Services;

public class HelloWorld : WebService {

     [ WebMethod ]
     public String SayHelloWorld() {
          return "Hello World";
     }
}
</pre></div>

<P>
This file starts with an ASP+ directive <I>WebService</I>, and sets the language to C# (you could also set the language to VB or C). Next, it imports the namespace System.Web.Services. This namespace is needed and you must include it. Next, the class <B>HelloWorld</B> is declared. This class is derived from the base class <B>WebService</B>. Finally, any methods that will be accessible as part of the service have the custom attribute <B>[WebMethod]</B> (or <"WebMethod>" in Visual Basic) in front of their signatures.
</P>
<P>
To make this service available, we might name the file HelloWorld.asmx and place it on a server called <B>Foo</B> inside a virtual directory called <B>Bar</B>. With Internet Explorer 5, you could then enter the URL <B>http://foo/bar/HelloWorld.asmx</B> and the resulting page would show the public methods for this Web Service (those marked with the WebMethod attribute), as well which protocols (such as SOAP, or HTTP GET) you can use to invoke these methods. Entering <B>http://foo/bar/HelloWorld.asmx?SDL</B> into the Internet Explorer address location produces the same information as an XML file, based on the Service Description Language (SDL) grammar. This SDL file is very important, and is used by clients that will access the service.
</P>
<H5>Accessing Web Services</H5>
<P>
Besides technology that allows developers to create Web Services, NGWS provides a sophisticated set of tools and code to consume, or access as a client, Web Services. Because Web Services are based on open protocols such as the Simple Object Access Protocol (SOAP) and HTTP, this client technology can also be used to consume non-ASP+ Web Services. Of course, the level of integration between ASP+ Web Services and the client technology is extremely high.
</P>
<P>
With the SDK, there is a tool called WebServiceUtil.exe. This program can be used to download the SDL description of a Web Service, and create a proxy class that addresses this service. For example, you could enter: 

<div class="code"><pre>
WebServiceUtil /c:proxy /pa:http://someDomain.com/someFolder/HelloWorld.asmx?SDL
</pre></div>

A Proxy class called HelloWorld.cs would then be created. 
</P>
<P>This class would look very similar to the class created in the previous section. It would contain a method called <B>SayHelloWorld</B> that returns a String. Compiling this proxy class into an application, and then calling this proxy classes method results in the proxy class packaging a SOAP request across HTTP, and take the SOAP-encoded response, and marshall it as a string.
</P>
<P>From the client perspective, the code would be simple:

<div class="code"><pre>
HelloWorld myHelloWorld = new HelloWorld();
String return = myHelloWorld.SayHelloWorld();
</pre></div>

And the return would be "Hello World".
</p>
<P>
The rest of this section deals with more advanced topics with Web Services, such as sending and receiving complex datatypes. There is also a section on Text Pattern Matching, which is an exciting technology to address <i>any</I> URI that returns text as if it was a Web Service. You can also perform data binding operations with Web Services, and that is discussed in the Data section.
<!-- #include virtual="/quickstart/aspplus/include/footer.inc" -->