<%@ Register TagPrefix="Acme" TagName="SourceRef" Src="/quickstart/util/SrcRef.aspx"%>

<!-- #include virtual="/quickstart/aspplus/include/header.inc" -->

<h4>Web Service Type Marshalling</h4>
This section illustrates that various data types that can be passed to and returned by Web Service methods.  
Because the Web Services implementation is built on top of the XML Serialization architecture, it supports a significant number of data types.
The following table lists the supported data types for Web Service methods when using the SOAP protocol (e.g. using the proxy generated 
by WebServiceUtil.exe).
<p>

<div class="indent">
<table class="table2" cellpadding="3" width="90%">
</tr>
<tr><th width="180"><b>Type</b></th><th><b>Description</b></th></tr>
<tr><td><b>Primitive Types</b></td><td>Standard primitive types.  The complete list of supported primitives are
String, Int32, Byte, Boolean, Int16, Int64, Single, Double, Decimal, DateTime (as XML's timeInstant), DateTime (as XML's date), DateTime (as XML's time), and XmlQualifiedName (as XML's QName).
</td></tr>
<tr><td><b>Enum Types</b></td><td>Enumeration types, e.g. "public enum color { red=1, blue=2 }"</td></tr>
<tr><td><b>Arrays of Primitives, Enums</b></td><td>Arrays of the above primitives, such as string[] and int[]</td></tr>
<tr><td><b>Classes and Structs</b></td><td>Class and struct types with public fields or properties.  The public properties and fields are serialized.</td></tr>
<tr><td><b>Arrays of Classes (Structs)</b></td><td>Arrays of the above.</td></tr>
<tr><td><b>DataSet</b></td><td>ADO+ DataSet Types (see the next section for an example).  DataSets can also appear as fields in structs or classes.
<p>
<b>Note:</b> Microsoft Visual Studio 7 & the XSD.EXE SDK utility have support for "strong-typing" a DataSet.  
These tools generate a class which inherits from DataSet to produce DataSet1, adding several methods/properties/etc that are specific 
to a particular XML schema.  If you pass DataSet, Web Services always transmits the schema along with the data 
(so it knows what tables & columns you are passing), and their types (e.g. int, string).  If you pass a subclass of DataSet 
(for example, DataSet1), Web Services assumes you are adding tables / columns in the constructor, and assumes that those 
tables/columns represent your schema.  In this case, it includes the schema in the SDL file, but is not passed every time you 
pass the DataSet instance.
</td></tr>
<tr><td><b>Arrays of DataSet</td><td>Arrays of the above.</b></td></tr>
<tr><td><b>XmlNode</b></td><td>
XmlNode is an in-memory representation of an XML fragment (like a lightweight XML document object model).  For example, 
"&lt;comment&gt;This is&lt;b&gt;pretty&lt;/b&gt; neat&lt;/comment&gt;" could be stored in an XmlNode.  You can pass 
XmlNodes as parameters, and they are added to the rest of the XML being passed to the Web Service (the other parameters) in a 
SOAP-compliant manner.   The same is true for return values.  This allows you to pass or return XML whose structure changes from 
call to call, or where you may not know all the types being passed.  XmlNode can also appear as fields in structs or classes.
</td></tr>
<tr><td><b>Arrays of XmlNode</b></td><td>Arrays of above.</td>
</table>
</div>
<p>

<div class="indent">
<b>Return values: </b><p>Whether calling a Web Service via SOAP or HTTP GET/POST all of the above types are supported for return values.  
<p>
<b>Parameters: </b><p>Both by-value and by-reference (in/out) parameters are supported when using the SOAP protocol.  By-reference parameters have the effect of sending the 
value both ways: up to the server, and back to the client.  When passing input parameters to a Web Service via HTTP GET/POST, only a 
limited set of data types are supported, and they must be by-value parameters.  The supported types for HTTP GET/POST parameters are listed below:
<p>
</div>

<div class="indent">
<table class="table2" width="90%">
</tr>
<tr><th width="180"><b>Type</b></th><th><b>Description</b></th></tr>
<tr><td><b>Primitive Types (limited)</b></td><td>Most standard primitive types.  The complete list of supported primitives are
Int32, String, Int16, Int64, Boolean, Single, Double, Decimal, DateTime, TimeSpan, UInt16, UInt32, UInt64, and Currency.
From the client's perspective, all these types turn into string.
</td></tr>
<tr><td><b>Enum Types</b></td><td>Enumeration types, e.g. "public enum color { red=1, blue=2 }". 
From the client's perspective, enums become classes with a static const string for each value.
</td></tr>
<tr><td><b>Arrays of Primitives, Enums</b></td><td>Arrays of the above primitives, such as string[] and int[]</td></tr>
</table>
</div>

<p>
The following example demonstrates the use of the types listed above, using a SOAP proxy generated from WebServiceUtil.exe.  Note 
that because there is more than one public class defined in the .asmx file, we must specify which is to be treated as the WebService 
class using the "Class" attribute of the WebService directive:
<p>

<div class="code"><pre>
&lt;%@ WebService Language="C#" Class="DataTypes" %&gt;
</pre></div>

<p>

<table>
<tr>
<td>
<Acme:SourceRef 
  RunSample="/quickstart/aspplus/samples/services/DataTypes/DataTypes.asmx" 
  ViewSource="/quickstart/aspplus/samples/services/DataTypes/DataTypes.src"
  Icon="/quickstart/aspplus/images/datatypes_asmx.gif"
  Caption="DataTypes.asmx"
  runat="server" />
</td>
<td>
<Acme:SourceRef 
  RunSample="/quickstart/aspplus/samples/services/DataTypes/DataTypes.asmx?SDL" 
  Icon="/quickstart/aspplus/images/service_sdl.gif"
  Caption="DataTypes.sdl"
  runat="server" />
</td>
</tr>
</table>

<p>

<UL>
<LI>The <B>SayHello</B> method shows returning a String from a service. 
<LI>The <B>SayHelloName</B> method returns a String, and also takes a String as a parameter.
<LI>The <B>GetIntArray</B> method shows how to return an array of integers.
<LI>The <B>GetMode</B> method returns an enum value.
<LI>The <B>GetOrder</B> method returns a class (which is almost the same as a struct here).
<LI>The <B>GetOrders</B> method returns an array of Order objects.
</UL>

<p>
Using the WebServiceUtil proxy generation tool, the marshalling of these data types is transparent to the consuming client application.  
A sample client application for the above Web Service follows:
<p>

<Acme:SourceRef 
  RunSample="/quickstart/aspplus/samples/services/DataTypes/DataTypesClient.aspx" 
  ViewSource="/quickstart/aspplus/samples/services/DataTypes/DataTypesClient.src"
  Icon="/quickstart/aspplus/images/datatypes_client.gif"
  Caption="DataTypesClient.aspx"
  runat="server" />

<!-- #include virtual="/quickstart/aspplus/include/footer.inc" -->